@@ -22,7 +22,11 @@ zbayes$methods(
                    verbose = 0, 
                    ..., 
                    data,
-                   by = NULL) {
+                   by = NULL,
+                   bootstrap = FALSE) {
+    if(!identical(bootstrap,FALSE)){
+      stop("Error: The bootstrap is not available for Markov chain Monte Carlo (MCMC) models.")
+    }
     .self$zelig.call <- match.call(expand.dots = TRUE)
     .self$model.call <- .self$zelig.call
     if (missing(verbose))
@@ -30,7 +34,7 @@ zbayes$methods(
 #     .self$model.call$family <- call(.self$family, .self$link)
     .self$model.call$verbose <- verbose
     .self$num <- mcmc # CC: check
-    callSuper(formula = formula, data = data, ..., by = by)
+    callSuper(formula = formula, data = data, ..., by = by, bootstrap = FALSE)
   }
 )
 
