@@ -31,8 +31,8 @@ zpoissongee$methods(
 
 
 zpoissongee$methods(
-  param = function(z.out) {
-    simparam.local <- callSuper(z.out)
+  param = function(z.out, method="mvn") {
+    simparam.local <- callSuper(z.out, method=method)
     return(simparam.local$simparam) # no ancillary parameter
   }
 )
